var Filters = Filters || {};

// Space for your helper functions
// ----------- STUDENT CODE BEGIN ------------
// ----------- Our reference solution uses 123 lines of code.
// ----------- STUDENT CODE END ------------

Filters.translation = function(mesh, x, y, z) {
    const t = new THREE.Vector3(x, y, z);

    const verts = mesh.getModifiableVertices();

    const n_vertices = verts.length;
    for (let i = 0; i < n_vertices; ++i) {
        verts[i].position.add(t);
    }

    mesh.calculateFacesArea();
    mesh.updateNormals();
};

// rotation: a Vector3 containing the desired rotation around each axis, in radians
Filters.rotation = function(mesh, x, y, z) {
    const verts = mesh.getModifiableVertices();

    // ----------- STUDENT CODE BEGIN ------------
    // ----------- Our reference solution uses 5 lines of code.
    // ----------- STUDENT CODE END ------------
    Gui.alertOnce ('Rotation is not implemented yet');

    mesh.calculateFacesArea();
    mesh.updateNormals();
};

Filters.scale = function(mesh, s) {
    const verts = mesh.getModifiableVertices();

    // ----------- STUDENT CODE BEGIN ------------
    // ----------- Our reference solution uses 4 lines of code.
    // ----------- STUDENT CODE END ------------
    Gui.alertOnce ('Scaling is not implemented yet');

    mesh.calculateFacesArea();
    mesh.updateNormals();
};

Filters.curvature = function(mesh) {
    // ----------- STUDENT CODE BEGIN ------------
    // ----------- Our reference solution uses 102 lines of code.
    // ----------- STUDENT CODE END ------------
    Gui.alertOnce ('Curvature is not implemented yet');
};

Filters.noise = function(mesh, factor) {
    const verts = mesh.getModifiableVertices();

    // ----------- STUDENT CODE BEGIN ------------
    // ----------- Our reference solution uses 13 lines of code.
    // ----------- STUDENT CODE END ------------
    Gui.alertOnce ('Noise is not implemented yet');

    mesh.calculateFacesArea();
    mesh.updateNormals();
};

// Note that the reference solution calls a giant utility function so the line
// count is not terrible representative of the true solution
//
// For implicit, you will want to compute I - M*L*delta, where I is the identity
// matrix, M is a diagonal "mass" matrix, and L is a Laplacian matrix. Then
// you will want to call math.lup() on your result in order to decompose the
// matrix. Finally, call math.lusolve() to compute the X,Y, and Z positions of
// vertices. Note that the decomposition step allows for fast solving multiple
// times. It would be possible to replace a few of these steps with simple matrix
// inversion; however, matrix inversion is far slower and less numerically stable
//
Filters.smooth = function(mesh, iter, delta, curvFlow, scaleDep, implicit) {
    const verts = mesh.getModifiableVertices();

    // ----------- STUDENT CODE BEGIN ------------
    // ----------- Our reference solution uses 22 lines of code.
    // ----------- STUDENT CODE END ------------
    Gui.alertOnce ('Smooth is not implemented yet');
    mesh.calculateFacesArea();
    mesh.updateNormals();
};

Filters.sharpen = function(mesh, iter, delta) {
    const verts = mesh.getModifiableVertices();

    // ----------- STUDENT CODE BEGIN ------------
    // ----------- Our reference solution uses 15 lines of code.
    // ----------- STUDENT CODE END ------------
    Gui.alertOnce ('Sharpen is not implemented yet');
    mesh.calculateFacesArea();
    mesh.updateNormals();
};

Filters.inflate = function(mesh, factor) {
    const verts = mesh.getModifiableVertices();

    // ----------- STUDENT CODE BEGIN ------------
    // ----------- Our reference solution uses 16 lines of code.
    // ----------- STUDENT CODE END ------------
    Gui.alertOnce ('Inflate is not implemented yet');

    mesh.calculateFacesArea();
    mesh.updateNormals();
};

Filters.twist = function(mesh, factor) {
    const verts = mesh.getModifiableVertices();

    // ----------- STUDENT CODE BEGIN ------------
    // ----------- Our reference solution uses 8 lines of code.
    // ----------- STUDENT CODE END ------------
    Gui.alertOnce ('Twist is not implemented yet');

    mesh.calculateFacesArea();
    mesh.updateNormals();
};

Filters.procFace = function(mesh, f) {
    const faceFlags = new Array(mesh.faces.length);
    for (let i = 0; i < mesh.faces.length; i++) {
        faceFlags[i] = 0;
    }
    let sum = f.area;
    const start_he = f.halfedge.opposite.next;
    let curr_he = start_he;
    do {
        if (faceFlags[curr_he.face.id] > 0) {
            break;
        }
        sum += curr_he.face.area;
        curr_he.face.selected = true;
        faceFlags[curr_he.face.id]++;
        const last_he = curr_he;
        curr_he = curr_he.opposite.next;
        if (curr_he.face == f) {
            curr_he = last_he.next.opposite.next;
        }
    } while (true);
};

Filters.wacky = function(mesh, factor) {
    // ----------- STUDENT CODE BEGIN ------------
    // ----------- Our reference solution uses 3 lines of code.
    // ----------- STUDENT CODE END ------------
    Gui.alertOnce ('Wacky is not implemented yet');

    mesh.calculateFacesArea();
    mesh.updateNormals();
};

Filters.triangulate = function(mesh) {
    const faces = mesh.getModifiableFaces();

    // ----------- STUDENT CODE BEGIN ------------
    // ----------- Our reference solution uses 4 lines of code.
    // ----------- STUDENT CODE END ------------
    Gui.alertOnce ('triangulate is not implemented yet');

    mesh.calculateFacesArea();
    mesh.updateNormals();
};

// wrapper for splitEdgeMakeVert in mesh.js
Filters.splitEdge = function(mesh) {
    const verts = mesh.getSelectedVertices();

    if (verts.length === 2) {
        mesh.splitEdgeMakeVert(verts[0], verts[1], 0.5);
    } else {
        console.log(
            'ERROR: to use split edge, select exactly 2 adjacent vertices'
        );
    }

    mesh.calculateFacesArea();
    mesh.updateNormals();
};

// wrapper for joinEdgeKillVert in mesh.js
Filters.joinEdges = function(mesh) {
    const verts = mesh.getSelectedVertices();

    if (verts.length === 3) {
        let v0 = verts[0],
            v1 = verts[1],
            v2 = verts[2];

        const he01 = mesh.edgeBetweenVertices(v0, v1);
        const he12 = mesh.edgeBetweenVertices(v1, v2);

        if (he01) {
            if (he12) {
                mesh.joinEdgeKillVert(verts[0], verts[1], verts[2]);
            } else {
                mesh.joinEdgeKillVert(verts[1], verts[0], verts[2]);
            }
        } else {
            if (he12) {
                mesh.joinEdgeKillVert(verts[0], verts[2], verts[1]);
            } else {
                console.log(
                    'ERROR: to use join edge, select exactly 3 vertices such that one only has edges to the other two'
                );
            }
        }
    } else {
        console.log('ERROR: to use join edge, select exactly 3 vertices');
    }

    mesh.calculateFacesArea();
    mesh.updateNormals();
};

// wrapper for splitFaceMakeEdge in mesh.js
Filters.splitFace = function(mesh) {
    const verts = mesh.getSelectedVertices();
    const faces = mesh.getModifiableFaces();

    if (verts.length === 2 && faces.length === 1) {
        mesh.splitFaceMakeEdge(faces[0], verts[0], verts[1]);
    } else {
        console.log(
            'ERROR: to use split face, select exactly 1 face and 2 nonadjacent vertices on it'
        );
    }

    mesh.calculateFacesArea();
    mesh.updateNormals();
};

// wrapper for joinFaceKillEdge in mesh.js
Filters.joinFaces = function(mesh) {
    const verts = mesh.getSelectedVertices();
    const faces = mesh.getModifiableFaces();

    if (verts.length === 2 && faces.length === 2) {
        mesh.joinFaceKillEdge(faces[0], faces[1], verts[0], verts[1]);
    } else {
        console.log(
            'ERROR: to use split face, select exactly 2 adjacent faces the 2 vertices between them'
        );
    }

    mesh.calculateFacesArea();
    mesh.updateNormals();
};

Filters.extrude = function(mesh, factor) {
    const faces = mesh.getModifiableFaces();

    // ----------- STUDENT CODE BEGIN ------------
    // ----------- Our reference solution uses 38 lines of code.
    // ----------- STUDENT CODE END ------------
    Gui.alertOnce ('Extrude is not implemented yet');

    mesh.calculateFacesArea();
    mesh.updateNormals();
};

Filters.truncate = function(mesh, factor) {
    const verts = mesh.getModifiableVertices();

    // ----------- STUDENT CODE BEGIN ------------
    // ----------- Our reference solution uses 64 lines of code.
    // ----------- STUDENT CODE END ------------
    Gui.alertOnce ('Truncate is not implemented yet');

    mesh.calculateFacesArea();
    mesh.updateNormals();
};

Filters.splitLong = function(mesh, factor) {
    // ----------- STUDENT CODE BEGIN ------------
    // ----------- Our reference solution uses 42 lines of code.
    // ----------- STUDENT CODE END ------------
    Gui.alertOnce ('Split Long Edges is not implemented yet');

    mesh.calculateFacesArea();
    mesh.updateNormals();
};

Filters.triSubdiv = function(mesh, levels) {
    Filters.triangulate(mesh);

    for (let l = 0; l < levels; l++) {
        const faces = mesh.getModifiableFaces();
        // ----------- STUDENT CODE BEGIN ------------
        // ----------- Our reference solution uses 43 lines of code.
        // ----------- STUDENT CODE END ------------
        Gui.alertOnce ('Triangle subdivide is not implemented yet');
    }

    mesh.calculateFacesArea();
    mesh.updateNormals();
};

Filters.loop = function(mesh, levels) {
    Filters.triangulate(mesh);

    for (let l = 0; l < levels; l++) {
        const faces = mesh.getModifiableFaces();
        // ----------- STUDENT CODE BEGIN ------------
        // ----------- Our reference solution uses 123 lines of code.
        // ----------- STUDENT CODE END ------------
        Gui.alertOnce ('Triangle subdivide is not implemented yet');
    }

    mesh.calculateFacesArea();
    mesh.updateNormals();
};

Filters.quadSubdiv = function(mesh, levels) {
    for (let l = 0; l < levels; l++) {
        const faces = mesh.getModifiableFaces();
        // ----------- STUDENT CODE BEGIN ------------
        // ----------- Our reference solution uses 55 lines of code.
        // ----------- STUDENT CODE END ------------
        Gui.alertOnce ('Quad subdivide is not implemented yet');
    }

    mesh.calculateFacesArea();
    mesh.updateNormals();
};

Filters.catmullClark = function(mesh, levels) {
    for (let l = 0; l < levels; l++) {
        const faces = mesh.faces;
        // ----------- STUDENT CODE BEGIN ------------
        // ----------- Our reference solution uses 102 lines of code.
        // ----------- STUDENT CODE END ------------
        Gui.alertOnce ('Catmull-Clark subdivide is not implemented yet');
    }

    mesh.calculateFacesArea();
    mesh.updateNormals();
};

Filters.parseSelected = function(sel) {
    if (sel === undefined || sel.replace === undefined) {
        return [];
    }
    if (typeof sel === 'number') {
        return [sel];
    }
    // sel = sel.replace(/[\(\)]/g,'');
    sel = sel.split(',');
    const parsedSel = [];
    for (let i = 0; i < sel.length; i++) {
        const idx = parseInt(sel[i]);
        if (!isNaN(idx)) {
            parsedSel.push(idx);
        }
    }
    return parsedSel;
};

// internal filter for updating selection
Filters.selection = function(mesh, vertIdxs, faceIdxs) {
    mesh.setSelectedVertices(Filters.parseSelected(vertIdxs));
    mesh.setSelectedFaces(Filters.parseSelected(faceIdxs));
};

// internal filter for setting display settings
Filters.displaySettings = function(
    mesh,
    showLabels,
    showHalfedge,
    shading,
    showVN,
    showFN,
    showGrid,
    showVertDots,
    showAxes,
    showVC,
    meshColor
) {
    Main.displaySettings.showIdLabels = showLabels;
    Main.displaySettings.wireframe = showHalfedge;
    Main.displaySettings.shading = shading;
    Main.displaySettings.showVN = showVN;
    Main.displaySettings.showFN = showFN;
    Main.displaySettings.showGrid = showGrid;
    Main.displaySettings.showVertDots = showVertDots;

    Main.displaySettings.showAxes = showAxes;
    Main.displaySettings.showVC = showVC;
    // Main.displaySettings.meshColor = meshColor;

    // Main.refreshDisplaySettings();
};
